{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TemplateHaskell #-}
{-# LANGUAGE QuasiQuotes #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DeriveAnyClass #-}

module SEARX where

import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)

import Language.JavaScript.Inline

import System.Directory
import System.IO.Temp
import System.Process

import Servant.Client

import SEARX.Client

import Data.Foldable
import qualified Data.Text as T

{-
debug purpose, use getMetadataWith
-}
getMetadataWith2 :: T.Text -> IO (Either ClientError Documents)
getMetadataWith2 q = do
  manager' <- newManager tlsManagerSettings
  runClientM
    (search (Just q) (Just 1) (Just 3) (Just "") (Just All) (Just Json))
    (mkClientEnv manager' $ BaseUrl Https "search.iscpif.fr" 443 "")

parseWebsite :: T.Text -> IO T.Text
parseWebsite url =
  withSystemTempDirectory "" $ \tmpdir -> do
    withCurrentDirectory tmpdir $
      traverse_
        callCommand
        [ "npm init --yes"
        , "npm install --save readability"
        -- , "npm install --save node-readability"
        ]
    withJSSession
      defJSSessionOpts {nodeWorkDir = Just tmpdir}
      [block|
          const read = (await import("readability")).default,
            reader = new Readability("<body>bal<title>title</title></body>").parse();

          return reader.title;


            |]

specConcatEith :: Semigroup a => Either b a -> Either b a -> Either b a
specConcatEith (Left _) b = b
specConcatEith a (Left _) = a
specConcatEith (Right a) (Right b) = Right $ a <> b


getMetadataWith :: T.Text -> Int -> IO (Either ClientError Documents)
getMetadataWith q i = do
  manager' <- newManager tlsManagerSettings
  getMetadataWith' manager' q i 1
  where client' man = mkClientEnv man $ BaseUrl Https "search.iscpif.fr" 443 ""
        search' r' p' = search (Just r') (Just 1) (Just p') (Just "") (Just All) (Just Json)
        getMetadataWith' man req maxP curP
          | maxP >= curP = do
              val <- runClientM (search' req curP) (client' man)
              nextVal <- getMetadataWith' man req maxP (curP + 1)
              return $ specConcatEith val nextVal
          | otherwise = return (Right $ Documents 0 [])
